@@ -17,6 +17,7 @@ from ..model.manager import ModelManager
 from ..model.nlp.sentence import SentenceTransformerModel
 from ..tool.browser import BrowserToolSet, BrowserToolSettings
 from ..tool.code import CodeToolSet
+from ..tool.context import ToolSettingsContext
 from ..tool.database import DatabaseToolSet, DatabaseToolSettings
 from ..tool.manager import ToolManager
 from ..tool.math import MathToolSet
@@ -54,12 +55,22 @@ class OrchestratorLoader:
         self._participant_id = participant_id
         self._stack = stack
 
+    @property
+    def hub(self) -> HuggingfaceHub:
+        return self._hub
+
+    @property
+    def participant_id(self) -> UUID:
+        return self._participant_id
+
     async def from_file(
         self,
         path: str,
         *,
         agent_id: UUID | None,
         disable_memory: bool = False,
+        uri: str | None = None,
+        tool_settings: ToolSettingsContext | None = None,
     ) -> Orchestrator:
         _l = self._log_wrapper(self._logger)
 
@@ -95,7 +106,7 @@ class OrchestratorLoader:
                 "uri" in config["engine"]
             ), "No uri defined in engine section of configuration"
 
-            uri = config["engine"]["uri"]
+            uri = uri or config["engine"]["uri"]
             engine_config = config["engine"]
             enable_tools = (
                 engine_config["tools"] if "tools" in engine_config else None
@@ -238,6 +249,19 @@ class OrchestratorLoader:
             if database_config:
                 database_settings = DatabaseToolSettings(**database_config)
 
+            if tool_settings:
+                browser_settings = tool_settings.browser or browser_settings
+                database_settings = tool_settings.database or database_settings
+                extra = tool_settings.extra
+            else:
+                extra = None
+
+            tool_settings = ToolSettingsContext(
+                browser=browser_settings,
+                database=database_settings,
+                extra=extra,
+            )
+
             tool_format = None
             tool_format_str = tool_section.get("format")
             if tool_format_str:
@@ -246,18 +270,14 @@ class OrchestratorLoader:
             _l("Loaded agent from %s", path, is_debug=False)
 
             return await self.from_settings(
-                settings,
-                browser_settings=browser_settings,
-                database_settings=database_settings,
-                tool_format=tool_format,
+                settings, tool_settings=tool_settings, tool_format=tool_format
             )
 
     async def from_settings(
         self,
         settings: OrchestratorSettings,
         *,
-        browser_settings: BrowserToolSettings | None = None,
-        database_settings: DatabaseToolSettings | None = None,
+        tool_settings: ToolSettingsContext | None = None,
         tool_format: ToolFormat | None = None,
     ) -> Orchestrator:
         _l = self._log_wrapper(self._logger)
@@ -342,6 +362,15 @@ class OrchestratorLoader:
             settings.sentence_model_window_size,
         )
 
+        browser_settings = tool_settings.browser if tool_settings else None
+        database_settings = tool_settings.database if tool_settings else None
+
+        _l(
+            "Tool settings: browser=%s, database=%s",
+            browser_settings,
+            database_settings,
+        )
+
         available_toolsets = [
             BrowserToolSet(
                 settings=browser_settings or BrowserToolSettings(),
