@@ -10,13 +10,15 @@ from ....entities import (
     ToolCall,
     ToolCallContext,
     ToolCallResult,
+    ToolCallToken,
 )
 from ....event import Event, EventType
 from ....event.manager import EventManager
 from ....model.response.text import TextGenerationResponse
 from ....tool.manager import ToolManager
-from ....model.response.parsers.tool import ToolCallParser
+from ....model.response.parsers.tool import ToolCallResponseParser
 from ....cli import CommandAbortException
+from base64 import b64encode
 from dataclasses import asdict, is_dataclass
 from inspect import iscoroutine
 from json import dumps
@@ -47,7 +49,7 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
     _participant_id: UUID | None
     _session_id: UUID | None
     _parser_queue: Queue[Token | TokenDetail | Event] | None
-    _tool_parser: ToolCallParser | None
+    _tool_parser: ToolCallResponseParser | None
 
     def __init__(
         self,
@@ -84,7 +86,7 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
         self._tool_confirm_all = False
         self._parser_queue = Queue()
         self._tool_parser = (
-            ToolCallParser(self._tool_manager, self._event_manager)
+            ToolCallResponseParser(self._tool_manager, self._event_manager)
             if enable_tool_parsing and self._tool_manager
             else None
         )
@@ -93,6 +95,10 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
     def input_token_count(self) -> int:
         return self._response.input_token_count
 
+    @property
+    def output_token_count(self) -> int:
+        return self._response.output_token_count
+
     @property
     def can_think(self) -> bool:
         return self._response.can_think
@@ -225,6 +231,7 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
             tool_messages = []
             for e in result_events:
                 tool_result = e.payload["result"]
+                tool_output = tool_result.result
                 tool_messages.append(
                     Message(
                         role=MessageRole.ASSISTANT,
@@ -237,17 +244,23 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
                         ],
                     )
                 )
-                tool_output = e.payload["result"].result
+                tool_result_output = dumps(
+                    asdict(tool_output)
+                    if is_dataclass(tool_output)
+                    else tool_output,
+                    default=lambda o: (
+                        b64encode(o).decode()
+                        if isinstance(o, (bytes, bytearray))
+                        else str(o)
+                    )
+                )
                 tool_messages.append(
                     Message(
                         role=MessageRole.TOOL,
-                        name=e.payload["result"].name,
-                        arguments=e.payload["result"].arguments,
-                        content=dumps(
-                            asdict(tool_output)
-                            if is_dataclass(tool_output)
-                            else tool_output
-                        ),
+                        name=tool_result.name,
+                        arguments=tool_result.arguments,
+                        content=tool_result_output,
+                        tool_call_result=tool_result,
                     )
                 )
 
@@ -300,6 +313,8 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
 
         try:
             token = await self._response_iterator.__anext__()
+            if isinstance(token, ToolCallToken) and token.call:
+                self._calls.put(token.call)
         except StopAsyncIteration:
             if self._tool_parser:
                 for item in await self._tool_parser.flush():
@@ -485,6 +500,7 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
                         "token_id": token_id,
                         "model_id": self._engine_agent.engine.model_id,
                         "token": token_str,
+                        "token_type": type(item).__qualname__,
                         "step": self._step,
                     },
                 )
@@ -500,6 +516,8 @@ class OrchestratorResponse(AsyncIterator[Token | TokenDetail | Event]):
 
         if isinstance(item, str) and self._tool_parser:
             items = await self._tool_parser.push(item)
+            if not items:
+                return await self.__anext__()
         else:
             items = [item]
 
