@@ -1,7 +1,4 @@
-from dataclasses import replace
-
 from ...agent.orchestrator import Orchestrator
-from ...event import Event, EventType, TOOL_TYPES
 from ...cli import confirm, get_input, has_input
 from ...cli.commands.cache import cache_delete, cache_download
 from ...entities import (
@@ -11,16 +8,17 @@ from ...entities import (
     Token,
     ToolCallToken,
 )
-from ...event import EventStats
-from ...model.response.text import TextGenerationResponse
+from ...entities import GenerationSettings  # noqa: F401
+from ...event import Event, EventStats, EventType, TOOL_TYPES
+from ...model.criteria import KeywordStoppingCriteria  # noqa: F401
 from ...model.hubs.huggingface import HuggingfaceHub
 from ...model.manager import ModelManager
-from ...entities import GenerationSettings  # noqa: F401
-from ...model.criteria import KeywordStoppingCriteria  # noqa: F401
 from ...model.nlp.sentence import SentenceTransformerModel
 from ...model.nlp.text.generation import TextGenerationModel
+from ...model.response.text import TextGenerationResponse
 from ...secrets import KeyringSecrets
 from . import get_model_settings
+from dataclasses import replace
 from argparse import Namespace
 from asyncio import (
     as_completed,
@@ -178,12 +176,15 @@ async def model_run(
         with manager.load(**model_settings) as model:
             logger.debug("Loaded model %s", model.config.__repr__())
 
+            tty_path = getattr(args, "tty", "/dev/tty") or "/dev/tty"
+
             if operation.requires_input or has_input(console):
                 input_string = get_input(
                     console,
                     _i["user_input"] + " ",
                     echo_stdin=not args.no_repl,
                     is_quiet=args.quiet,
+                    tty_path=tty_path,
                 )
                 if not input_string:
                     return
@@ -371,6 +372,8 @@ async def token_generation(
             console.print(text_token, end="")
         return
 
+    stop_signal = EventSignal()
+
     # From here on, display includes stats and may include token probabilities
 
     if not orchestrator or (
@@ -395,59 +398,57 @@ async def token_generation(
                 display_tokens=display_tokens,
                 dtokens_pick=dtokens_pick,
                 refresh_per_second=refresh_per_second,
-                stop_signal=None,
+                stop_signal=stop_signal,
                 tool_events_limit=tool_events_limit,
                 with_stats=with_stats,
             )
-        return
+    else:
+        events_height = 6
+        tools_height = 10
+        empty = ""
+        group = Group(empty, empty, empty)
+        events_group_index = 0
+        tools_group_index = 1
+        tokens_group_index = 2
 
-    stop_signal = EventSignal()
-    events_height = 6
-    tools_height = 10
-    empty = ""
-    group = Group(empty, empty, empty)
-    events_group_index = 0
-    tools_group_index = 1
-    tokens_group_index = 2
-
-    with Live(
-        group, refresh_per_second=refresh_per_second, screen=args.record
-    ) as live:
-        await gather(
-            _event_stream(
-                args,
-                console,
-                live,
-                group,
-                events_group_index,
-                tools_group_index,
-                orchestrator,
-                theme,
-                events_height=events_height,
-                tools_height=tools_height,
-                stop_signal=stop_signal,
-            ),
-            _token_stream(
-                args,
-                console,
-                live,
-                group,
-                tokens_group_index,
-                theme,
-                logger,
-                orchestrator,
-                event_stats,
-                lm,
-                input_string,
-                response,
-                display_tokens=display_tokens,
-                dtokens_pick=dtokens_pick,
-                refresh_per_second=refresh_per_second,
-                stop_signal=stop_signal,
-                tool_events_limit=tool_events_limit,
-                with_stats=with_stats,
-            ),
-        )
+        with Live(
+            group, refresh_per_second=refresh_per_second, screen=args.record
+        ) as live:
+            await gather(
+                _event_stream(
+                    args,
+                    console,
+                    live,
+                    group,
+                    events_group_index,
+                    tools_group_index,
+                    orchestrator,
+                    theme,
+                    events_height=events_height,
+                    tools_height=tools_height,
+                    stop_signal=stop_signal,
+                ),
+                _token_stream(
+                    args,
+                    console,
+                    live,
+                    group,
+                    tokens_group_index,
+                    theme,
+                    logger,
+                    orchestrator,
+                    event_stats,
+                    lm,
+                    input_string,
+                    response,
+                    display_tokens=display_tokens,
+                    dtokens_pick=dtokens_pick,
+                    refresh_per_second=refresh_per_second,
+                    stop_signal=stop_signal,
+                    tool_events_limit=tool_events_limit,
+                    with_stats=with_stats,
+                ),
+            )
 
 
 async def _event_stream(
@@ -521,7 +522,8 @@ async def _token_stream(
     tool_events_limit: int | None,
     with_stats: bool = True,
 ) -> None:
-    display_time_to_n_token = args.display_time_to_n_token or 256
+    display_time_to_n_token = args.display_time_to_n_token
+    display_reasoning_time = not getattr(args, "skip_display_reasoning_time")
     display_pause = (
         args.display_pause
         if args.display_pause and args.display_pause > 0
@@ -537,12 +539,12 @@ async def _token_stream(
     tool_events: list[Event] = []
     tool_event_calls: list[Event] = []
     tool_event_results: list[Event] = []
+    completed_call_ids: set[str] = set()
     total_tokens = 0
     frame_minimum_pause_ms = (
         100 if display_pause > 0 and display_tokens > 0 else 0
     )
 
-    start = perf_counter()
     input_token_count = (
         response.input_token_count
         if response.input_token_count
@@ -554,14 +556,19 @@ async def _token_stream(
     )
     ttft: float | None = None
     ttnt: float | None = None
-    token_frame_list: list[tuple[Token | None, RenderableType]] = None
     last_current_dtoken: Token | None = None
     tool_running_spinner: Spinner | None = None
 
     if start_thinking and response.can_think and not response.is_thinking:
         response.set_thinking(start_thinking)
 
+    start = perf_counter()
+    started_reasoning = perf_counter() if response.is_thinking else None
+    reasoning_time = None
+
     async for token in response:
+        is_reasoning_token = isinstance(token, ReasoningToken)
+
         if isinstance(token, Event):
             event = token
             tool_events.append(event)
@@ -576,14 +583,25 @@ async def _token_stream(
                     input_token_count = inner_response.input_token_count
             elif event.type == EventType.TOOL_RESULT:
                 tool_event_results.append(event)
+                if "call" in event.payload:
+                    completed_call_ids.add(event.payload["call"].id)
             else:
                 tool_event_calls.append(event)
-
         else:
+            if (
+                display_reasoning_time
+                and not is_reasoning_token
+                and started_reasoning is not None
+            ):
+                reasoning_time = perf_counter() - started_reasoning
+                started_reasoning = None
+
             text_token = token.token if isinstance(token, Token) else token
             if isinstance(token, ToolCallToken):
                 tool_text_tokens.append(text_token)
-            elif isinstance(token, ReasoningToken):
+            elif is_reasoning_token:
+                if not started_reasoning:
+                    started_reasoning = perf_counter()
                 thinking_text_tokens.append(text_token)
             else:
                 answer_text_tokens.append(text_token)
@@ -594,12 +612,7 @@ async def _token_stream(
                 c.name
                 for e in tool_event_calls
                 for c in e.payload
-                if c.id
-                not in {
-                    r.payload["call"].id
-                    for r in tool_event_results
-                    if r.type == EventType.TOOL_RESULT and "call" in r.payload
-                }
+                if c.id not in completed_call_ids
             ]
 
             tool_running_spinner = (
@@ -619,15 +632,28 @@ async def _token_stream(
                 else None
             )
 
-        total_tokens = total_tokens + 1
         elapsed = perf_counter() - start
+        total_tokens += 1
+
         if ttft is None:
             ttft = elapsed
-        if ttnt is None and total_tokens >= display_time_to_n_token:
+        if (
+            ttnt is None
+            and display_time_to_n_token
+            and total_tokens >= display_time_to_n_token
+        ):
             ttnt = elapsed
 
+        ttsr = None
+        if display_reasoning_time and reasoning_time:
+            ttsr = reasoning_time
+
         if display_tokens and isinstance(token, Token):
             tokens.append(token)
+        limit_answer_height = not getattr(
+            args, "display_answer_height_expand", False
+        )
+        answer_height = getattr(args, "display_answer_height", 12)
 
         token_frames_promise = theme.tokens(
             lm.model_id,
@@ -673,12 +699,14 @@ async def _token_stream(
             tool_running_spinner,
             ttft,
             ttnt,
+            ttsr,
             elapsed,
             console.width,
             logger,
             event_stats,
+            height=answer_height,
             tool_events_limit=tool_events_limit,
-            limit_answer_height=True,
+            limit_answer_height=limit_answer_height,
             maximum_frames=1,
             start_thinking=start_thinking,
         )
@@ -687,9 +715,6 @@ async def _token_stream(
             token_frame async for token_frame in token_frames_promise
         ]
 
-        # We prioritize a single selected dtoken at a time, it being
-        # the leftmost  selected which is also guaranteed by setting
-        # minimum_frames=1 when calling theme.tokens()
         token_frames = [token_frame_list[0]]
 
         for current_dtoken, frame in token_frames:
@@ -712,21 +737,21 @@ async def _token_stream(
             ):
                 await sleep(display_pause / 1000)
 
-    if (
-        dtokens_pick > 0
-        and args.display_probabilities
-        and token_frame_list
-        and len(token_frame_list) > 0
-    ):
-        for current_dtoken, frame in token_frame_list[1:]:
-            _render_frame(
-                args, console, live, frame, group, tokens_group_index
-            )
+        if (
+            dtokens_pick > 0
+            and args.display_probabilities
+            and token_frame_list
+            and len(token_frame_list) > 0
+        ):
+            for current_dtoken, frame in token_frame_list[1:]:
+                _render_frame(
+                    args, console, live, frame, group, tokens_group_index
+                )
 
-            if current_dtoken and display_pause > 0:
-                await sleep(display_pause / 1000)
-            elif frame_minimum_pause_ms > 0:
-                await sleep(frame_minimum_pause_ms / 1000)
+                if current_dtoken and display_pause > 0:
+                    await sleep(display_pause / 1000)
+                elif frame_minimum_pause_ms > 0:
+                    await sleep(frame_minimum_pause_ms / 1000)
 
     if stop_signal:
         stop_signal.set()
