@@ -41,6 +41,9 @@ class ObjectDetectionModel(ImageClassificationModel):
             revision=self._revision,
             device_map=self._device,
             tp_plan=Engine._get_tp_plan(self._settings.parallel),
+            distributed_config=Engine._get_distributed_config(
+                self._settings.distributed_config
+            ),
         )
         return model
 
