@@ -40,6 +40,9 @@ class ImageToTextModel(TransformerModel):
             self._model_id,
             device_map=self._device,
             tp_plan=Engine._get_tp_plan(self._settings.parallel),
+            distributed_config=Engine._get_distributed_config(
+                self._settings.distributed_config
+            ),
         )
         return model
 
@@ -99,6 +102,9 @@ class ImageTextToTextModel(ImageToTextModel):
             torch_dtype=Engine.weight(self._settings.weight_type),
             device_map=self._device,
             tp_plan=Engine._get_tp_plan(self._settings.parallel),
+            distributed_config=Engine._get_distributed_config(
+                self._settings.distributed_config
+            ),
         )
         return model
 
