@@ -2,11 +2,12 @@ from ..agent.loader import OrchestratorLoader
 from ..agent.orchestrator import Orchestrator
 from ..entities import OrchestratorSettings
 from ..model.hubs.huggingface import HuggingfaceHub
-from ..tool.browser import BrowserToolSettings
-from ..tool.database import DatabaseToolSettings
+from ..tool.context import ToolSettingsContext
 from ..utils import logger_replace
+from .entities import OrchestratorContext
 from contextlib import AsyncExitStack, asynccontextmanager
 from fastapi import APIRouter, FastAPI, Request
+from fastapi.middleware.cors import CORSMiddleware
 from logging import Logger
 from typing import TYPE_CHECKING
 from uuid import UUID, uuid4
@@ -24,13 +25,17 @@ def agents_server(
     reload: bool,
     specs_path: str | None,
     settings: OrchestratorSettings | None,
-    browser_settings: BrowserToolSettings | None,
-    database_settings: DatabaseToolSettings | None,
+    tool_settings: ToolSettingsContext | None,
     prefix_mcp: str,
     prefix_openai: str,
     logger: Logger,
     agent_id: UUID | None = None,
     participant_id: UUID | None = None,
+    allow_origins: list[str] | None = None,
+    allow_origin_regex: str | None = None,
+    allow_methods: list[str] | None = None,
+    allow_headers: list[str] | None = None,
+    allow_credentials: bool = False,
 ) -> "Server":
     """Build a configured Uvicorn server for Avalan agents.
 
@@ -46,13 +51,17 @@ def agents_server(
         reload: Whether Uvicorn should reload on changes.
         specs_path: Optional path to an agent specification file.
         settings: Optional in-memory orchestrator settings.
-        browser_settings: Optional browser tool configuration.
-        database_settings: Optional database tool configuration.
+        tool_settings: Optional tool configuration context.
         prefix_mcp: URL prefix for MCP endpoints.
         prefix_openai: URL prefix for OpenAI-compatible endpoints.
         logger: Application logger.
         agent_id: Optional agent identifier.
         participant_id: Optional participant identifier.
+        allow_origins: Optional list of allowed CORS origins.
+        allow_origin_regex: Optional regex for allowed CORS origins.
+        allow_methods: Optional list of allowed CORS methods.
+        allow_headers: Optional list of allowed CORS headers.
+        allow_credentials: Whether to allow CORS credentials.
 
     Returns:
         Configured Uvicorn server instance.
@@ -62,6 +71,8 @@ def agents_server(
     ), "Provide either specs_path or settings, but not both"
 
     from ..server.routers import chat
+    from ..server.routers import engine
+    from ..server.routers import responses
     from mcp.server.lowlevel.server import Server as MCPServer
     from mcp.server.sse import SseServerTransport
     from mcp.types import EmbeddedResource, ImageContent, TextContent, Tool
@@ -75,36 +86,68 @@ def agents_server(
         environ["TOKENIZERS_PARALLELISM"] = "false"
         async with AsyncExitStack() as stack:
             logger.info("Loading OrchestratorLoader in app lifespan")
+            pid = participant_id or uuid4()
             loader = OrchestratorLoader(
                 hub=hub,
                 logger=logger,
-                participant_id=participant_id or uuid4(),
+                participant_id=pid,
                 stack=stack,
             )
-            if specs_path:
-                orchestrator = await loader.from_file(
-                    specs_path,
+            tool_ctx = tool_settings
+            ctx = OrchestratorContext(
+                participant_id=pid,
+                specs_path=specs_path,
+                settings=settings,
+                tool_settings=tool_ctx,
+            )
+            app.state.ctx = ctx
+            app.state.stack = stack
+            app.state.loader = loader
+            if ctx.specs_path:
+                orchestrator_cm = await loader.from_file(
+                    ctx.specs_path,
                     agent_id=agent_id or uuid4(),
+                    tool_settings=ctx.tool_settings,
                 )
             else:
-                orchestrator = await loader.from_settings(
-                    settings,
-                    browser_settings=browser_settings,
-                    database_settings=database_settings,
+                orchestrator_cm = await loader.from_settings(
+                    ctx.settings,
+                    tool_settings=ctx.tool_settings,
                 )
-            orchestrator = await stack.enter_async_context(orchestrator)
+            orchestrator = await stack.enter_async_context(orchestrator_cm)
             di_set(app, logger=logger, orchestrator=orchestrator)
+            app.state.agent_id = orchestrator.id
             logger.info(
                 "Agent loaded from %s in app lifespan",
-                specs_path if specs_path else "inline settings",
+                ctx.specs_path if ctx.specs_path else "inline settings",
             )
             yield
 
     logger.debug("Creating %s server", name)
     app = FastAPI(title=name, version=version, lifespan=lifespan)
 
+    if any(
+        [
+            allow_origins,
+            allow_origin_regex,
+            allow_methods,
+            allow_headers,
+            allow_credentials,
+        ]
+    ):
+        app.add_middleware(
+            CORSMiddleware,
+            allow_origins=allow_origins or [],
+            allow_origin_regex=allow_origin_regex,
+            allow_credentials=allow_credentials,
+            allow_methods=allow_methods or ["*"],
+            allow_headers=allow_headers or ["*"],
+        )
+
     logger.debug("Adding routes to %s server", name)
     app.include_router(chat.router, prefix=prefix_openai)
+    app.include_router(responses.router, prefix=prefix_openai)
+    app.include_router(engine.router)
 
     logger.debug("Creating MCP server with SSE")
     mcp_server = MCPServer(name=name)
