@@ -1,10 +1,21 @@
-from ..entities import MessageRole
+from ..entities import MessageRole, OrchestratorSettings
+from ..tool.context import ToolSettingsContext
+from dataclasses import dataclass
 from pydantic import BaseModel, Field
 from typing import Annotated, Literal
+from uuid import UUID
 
 JSONType = Literal["bool", "float", "int", "object", "string"]
 
 
+@dataclass(kw_only=True, frozen=True)
+class OrchestratorContext:
+    participant_id: UUID
+    specs_path: str | None = None
+    settings: OrchestratorSettings | None = None
+    tool_settings: ToolSettingsContext | None = None
+
+
 class ResponseFormatText(BaseModel):
     type: Literal["text"]
 
@@ -147,6 +158,22 @@ class ChatCompletionRequest(BaseModel):
     tool_choice: Literal["auto", "none", "required"] | str | dict | None = None
 
 
+class ResponsesRequest(BaseModel):
+    model: str
+    input: list[ChatMessage] = Field(...)
+    temperature: float | None = 1.0
+    top_p: float | None = 1.0
+    n: int | None = 1
+    stream: bool | None = False
+    stop: str | list[str] | None = None
+    max_tokens: int | None = None
+    response_format: ResponseFormat | None = None
+
+    @property
+    def messages(self) -> list[ChatMessage]:  # type: ignore[override]
+        return self.input
+
+
 class ChatCompletionChunkChoiceDelta(BaseModel):
     content: str
 
@@ -196,3 +223,8 @@ class ModelInfo(BaseModel):
 class ModelList(BaseModel):
     object: str = "list"
     data: list[ModelInfo]
+
+
+class EngineRequest(BaseModel):
+    uri: str
+    database: str | None = None
