@@ -1,13 +1,7 @@
+from . import orchestrate
 from .. import di_get_logger, di_get_orchestrator
 from ...agent.orchestrator import Orchestrator
-from ...entities import (
-    GenerationSettings,
-    Message,
-    MessageContentImage,
-    MessageContentText,
-    MessageRole,
-    ReasoningToken,
-)
+from ...entities import MessageRole, ReasoningToken, ToolCallToken
 from ...event import Event
 from ...server.entities import (
     ChatCompletionChoice,
@@ -18,14 +12,10 @@ from ...server.entities import (
     ChatCompletionResponse,
     ChatMessage,
     ChatCompletionUsage,
-    ContentImage,
-    ContentText,
 )
 from fastapi import APIRouter, Depends
 from fastapi.responses import StreamingResponse
 from logging import Logger
-from time import time
-from uuid import uuid4
 
 router = APIRouter(
     prefix="/chat",
@@ -51,51 +41,10 @@ async def create_chat_completion(
         "Processing chat completion request with messages %r", request
     )
 
-    messages = [
-        Message(role=req.role, content=_to_message_content(req.content))
-        for req in request.messages
-    ]
-
-    logger.debug(
-        "Transformed chat completion request to engine input %r", messages
+    response, response_id, timestamp = await orchestrate(
+        request, logger, orchestrator
     )
 
-    if request.stream and (request.n or 1) > 1:
-        raise ValueError("Streaming multiple completions is not supported")
-
-    response_id = uuid4()
-    timestamp = int(time())
-
-    settings = GenerationSettings(
-        use_async_generator=request.stream,
-        temperature=request.temperature,
-        max_new_tokens=request.max_tokens,
-        stop_strings=request.stop,
-        top_p=request.top_p,
-        num_return_sequences=request.n,
-        response_format=(
-            request.response_format.model_dump(
-                by_alias=True, exclude_none=True
-            )
-            if request.response_format
-            else None
-        ),
-    )
-
-    logger.debug(
-        "Calling orchestrator with input %r and settings %r for response %s",
-        messages,
-        settings,
-        response_id,
-    )
-
-    logger.info(
-        "Calling orchestrator %s for chat completion request",
-        str(orchestrator),
-    )
-
-    response = await orchestrator(messages, settings=settings)
-
     logger.info(
         "Orchestrator %s responded for chat completion request",
         str(orchestrator),
@@ -108,8 +57,7 @@ async def create_chat_completion(
             async for token in response:
                 if isinstance(token, Event):
                     continue
-
-                if isinstance(token, ReasoningToken):
+                elif isinstance(token, (ReasoningToken, ToolCallToken)):
                     token = token.token
 
                 choice = ChatCompletionChunkChoice(
@@ -131,7 +79,13 @@ async def create_chat_completion(
         )
 
         return StreamingResponse(
-            generate_chunks(), media_type="text/event-stream"
+            generate_chunks(),
+            media_type="text/event-stream",
+            headers={
+                "Cache-Control": "no-cache",
+                "Connection": "keep-alive",
+                "X-Accel-Buffering": "no",
+            },
         )
 
     # Non streaming
@@ -159,19 +113,3 @@ async def create_chat_completion(
     await orchestrator.sync_messages()
 
     return response
-
-
-def _to_message_content(item):
-    if isinstance(item, list):
-        return [
-            _to_message_content(i)
-            for i in item
-            if isinstance(i, (ContentImage, ContentText, str))
-        ]
-    if isinstance(item, ContentImage):
-        return MessageContentImage(type=item.type, image_url=item.image_url)
-    if isinstance(item, ContentText):
-        return MessageContentText(type=item.type, text=item.text)
-    if isinstance(item, str):
-        return MessageContentText(type="text", text=item)
-    raise TypeError(f"Unsupported content type: {type(item).__name__}")
